@@ -1,4 +1,4 @@
-/* See LICENSE file for license and copyright information */
+/* SPDX-License-Identifier: Zlib */
 
 #include <string.h>
 #include <stdlib.h>
@@ -108,9 +108,9 @@ cmd_bookmark_open(girara_session_t* session, girara_list_t* argument_list)
   if (argc != 1) {
     GString* string = g_string_new(NULL);
 
-    GIRARA_LIST_FOREACH(zathura->bookmarks.bookmarks, zathura_bookmark_t*, iter, bookmark)
+    GIRARA_LIST_FOREACH_BODY(zathura->bookmarks.bookmarks, zathura_bookmark_t*, bookmark,
       g_string_append_printf(string, "<b>%s</b>: %u\n", bookmark->id, bookmark->page);
-    GIRARA_LIST_FOREACH_END(zathura->bookmarks.bookmarks, zathura_bookmark_t*, iter, bookmark);
+    );
 
     if (strlen(string->str) > 0) {
       g_string_erase(string, strlen(string->str) - 1, 1);
@@ -167,19 +167,21 @@ cmd_info(girara_session_t* session, girara_list_t* UNUSED(argument_list))
   }
 
   struct meta_field {
-    char* name;
+    const char* name;
     zathura_document_information_type_t field;
   };
 
   const struct meta_field meta_fields[] = {
     { _("Title"),             ZATHURA_DOCUMENT_INFORMATION_TITLE },
-    { _("Author"),            ZATHURA_DOCUMENT_INFORMATION_AUTHOR },
     { _("Subject"),           ZATHURA_DOCUMENT_INFORMATION_SUBJECT },
     { _("Keywords"),          ZATHURA_DOCUMENT_INFORMATION_KEYWORDS },
+    { _("Author"),            ZATHURA_DOCUMENT_INFORMATION_AUTHOR },
     { _("Creator"),           ZATHURA_DOCUMENT_INFORMATION_CREATOR },
     { _("Producer"),          ZATHURA_DOCUMENT_INFORMATION_PRODUCER },
     { _("Creation date"),     ZATHURA_DOCUMENT_INFORMATION_CREATION_DATE },
-    { _("Modification date"), ZATHURA_DOCUMENT_INFORMATION_MODIFICATION_DATE }
+    { _("Modification date"), ZATHURA_DOCUMENT_INFORMATION_MODIFICATION_DATE },
+    { _("Format"),            ZATHURA_DOCUMENT_INFORMATION_FORMAT },
+    { _("Other"),             ZATHURA_DOCUMENT_INFORMATION_OTHER }
   };
 
   girara_list_t* information = zathura_document_get_information(zathura->document, NULL);
@@ -190,18 +192,18 @@ cmd_info(girara_session_t* session, girara_list_t* UNUSED(argument_list))
 
   GString* string = g_string_new(NULL);
 
-  GIRARA_LIST_FOREACH(information, zathura_document_information_entry_t*, iter, entry)
-  if (entry != NULL) {
-    for (unsigned int i = 0; i < LENGTH(meta_fields); i++) {
-      if (meta_fields[i].field == entry->type) {
-        g_string_append_printf(string, "<b>%s:</b> %s\n", meta_fields[i].name, entry->value);
+  for (unsigned int i = 0; i < LENGTH(meta_fields); i++) {
+    GIRARA_LIST_FOREACH_BODY(information, zathura_document_information_entry_t*, entry,
+      if (entry != NULL) {
+        if (meta_fields[i].field == entry->type) {
+          g_string_append_printf(string, "<b>%s:</b> %s\n", meta_fields[i].name, entry->value);
+        }
       }
-    }
+    );
   }
-  GIRARA_LIST_FOREACH_END(information, zathura_document_information_entry_t*, iter, entry);
 
-  if (strlen(string->str) > 0) {
-    g_string_erase(string, strlen(string->str) - 1, 1);
+  if (string->len > 0) {
+    g_string_erase(string, string->len - 1, 1);
     girara_notify(session, GIRARA_INFO, "%s", string->str);
   } else {
     girara_notify(session, GIRARA_INFO, _("No information available."));
@@ -251,7 +253,7 @@ cmd_open(girara_session_t* session, girara_list_t* argument_list)
 
     document_open_idle(zathura, girara_list_nth(argument_list, 0),
                        (argc == 2) ? girara_list_nth(argument_list, 1) : NULL,
-                       ZATHURA_PAGE_NUMBER_UNSPECIFIED, NULL, NULL);
+                       ZATHURA_PAGE_NUMBER_UNSPECIFIED, NULL, NULL, NULL, NULL);
   } else {
     girara_notify(session, GIRARA_ERROR, _("No arguments given."));
     return false;
@@ -280,6 +282,11 @@ cmd_print(girara_session_t* session, girara_list_t* UNUSED(argument_list))
     return false;
   }
 
+  if (zathura->global.sandbox == ZATHURA_SANDBOX_STRICT) {
+    girara_notify(zathura->ui.session, GIRARA_ERROR, _("Printing is not permitted in strict sandbox mode"));
+    return false;
+  }
+
   print(zathura);
 
   return true;
@@ -288,12 +295,7 @@ cmd_print(girara_session_t* session, girara_list_t* UNUSED(argument_list))
 bool
 cmd_nohlsearch(girara_session_t* session, girara_list_t* UNUSED(argument_list))
 {
-  g_return_val_if_fail(session != NULL, false);
-  g_return_val_if_fail(session->global.data != NULL, false);
-  zathura_t* zathura = session->global.data;
-
-  document_draw_search_results(zathura, false);
-  render_all(zathura);
+  sc_nohlsearch(session, NULL, NULL, 0);
 
   return true;
 }
@@ -311,11 +313,7 @@ cmd_save(girara_session_t* session, girara_list_t* argument_list)
   }
 
   if (girara_list_size(argument_list) == 1) {
-    if (document_save(zathura, girara_list_nth(argument_list, 0), false) == true) {
-      girara_notify(session, GIRARA_INFO, _("Document saved."));
-    } else {
-      girara_notify(session, GIRARA_INFO, _("Failed to save document."));
-    }
+    document_save(zathura, girara_list_nth(argument_list, 0), false);
   } else {
     girara_notify(session, GIRARA_ERROR, _("Invalid number of arguments."));
     return false;
@@ -337,11 +335,7 @@ cmd_savef(girara_session_t* session, girara_list_t* argument_list)
   }
 
   if (girara_list_size(argument_list) == 1) {
-    if (document_save(zathura, girara_list_nth(argument_list, 0), true) == true) {
-      girara_notify(session, GIRARA_INFO, _("Document saved."));
-    } else {
-      girara_notify(session, GIRARA_INFO, _("Failed to save document."));
-    }
+    document_save(zathura, girara_list_nth(argument_list, 0), true);
   } else {
     girara_notify(session, GIRARA_ERROR, _("Invalid number of arguments."));
     return false;
@@ -532,10 +526,15 @@ cmd_exec(girara_session_t* session, girara_list_t* argument_list)
   g_return_val_if_fail(session->global.data != NULL, false);
   zathura_t* zathura = session->global.data;
 
+  if (zathura->global.sandbox == ZATHURA_SANDBOX_STRICT) {
+    girara_notify(zathura->ui.session, GIRARA_ERROR, _("Exec is not permitted in strict sandbox mode"));
+    return false;
+  }
+
   if (zathura->document != NULL) {
     const char* path = zathura_document_get_path(zathura->document);
 
-    GIRARA_LIST_FOREACH(argument_list, char*, iter, value)
+    GIRARA_LIST_FOREACH_BODY_WITH_ITER(argument_list, char*, iter, value,
       char* r = girara_replace_substring(value, "$FILE", path);
 
       if (r != NULL) {
@@ -546,7 +545,7 @@ cmd_exec(girara_session_t* session, girara_list_t* argument_list)
           girara_list_iterator_set(iter, s);
         }
       }
-    GIRARA_LIST_FOREACH_END(argument_list, char*, iter, value);
+    );
   }
 
   return girara_exec_with_argument_list(session, argument_list);
