@@ -1,4 +1,4 @@
-/* See LICENSE file for license and copyright information */
+/* SPDX-License-Identifier: Zlib */
 
 #include "config.h"
 #include "commands.h"
@@ -23,7 +23,7 @@
 
 static void
 cb_jumplist_change(girara_session_t* session, const char* name,
-                   girara_setting_type_t UNUSED(type), void* value, void* UNUSED(data))
+                   girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
 {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
@@ -46,7 +46,7 @@ cb_jumplist_change(girara_session_t* session, const char* name,
 
 static void
 cb_color_change(girara_session_t* session, const char* name,
-                girara_setting_type_t UNUSED(type), void* value, void* UNUSED(data))
+                girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
 {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
@@ -56,9 +56,11 @@ cb_color_change(girara_session_t* session, const char* name,
 
   const char* string_value = (const char*) value;
   if (g_strcmp0(name, "highlight-color") == 0) {
-    gdk_rgba_parse(&(zathura->ui.colors.highlight_color), string_value);
+    parse_color(&zathura->ui.colors.highlight_color, string_value);
+  } else if (g_strcmp0(name, "highlight-fg") == 0) {
+    parse_color(&zathura->ui.colors.highlight_color_fg, string_value);
   } else if (g_strcmp0(name, "highlight-active-color") == 0) {
-    gdk_rgba_parse(&(zathura->ui.colors.highlight_color_active), string_value);
+    parse_color(&zathura->ui.colors.highlight_color_active, string_value);
   } else if (g_strcmp0(name, "recolor-darkcolor") == 0) {
     if (zathura->sync.render_thread != NULL) {
       zathura_renderer_set_recolor_colors_str(zathura->sync.render_thread, NULL, string_value);
@@ -68,9 +70,9 @@ cb_color_change(girara_session_t* session, const char* name,
       zathura_renderer_set_recolor_colors_str(zathura->sync.render_thread, string_value, NULL);
     }
   } else if (g_strcmp0(name, "render-loading-bg") == 0) {
-    gdk_rgba_parse(&(zathura->ui.colors.render_loading_bg), string_value);
+    parse_color(&zathura->ui.colors.render_loading_bg, string_value);
   } else if (g_strcmp0(name, "render-loading-fg") == 0) {
-    gdk_rgba_parse(&(zathura->ui.colors.render_loading_fg), string_value);
+    parse_color(&zathura->ui.colors.render_loading_fg, string_value);
   }
 
   render_all(zathura);
@@ -78,31 +80,70 @@ cb_color_change(girara_session_t* session, const char* name,
 
 static void
 cb_nohlsearch_changed(girara_session_t* session, const char* UNUSED(name),
-                      girara_setting_type_t UNUSED(type), void* value, void* UNUSED(data))
+                      girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
 {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
   g_return_if_fail(session->global.data != NULL);
   zathura_t* zathura = session->global.data;
 
-  bool* bvalue = value;
+  const bool* bvalue = value;
   document_draw_search_results(zathura, !*bvalue);
   render_all(zathura);
 }
 
 static void
 cb_incsearch_changed(girara_session_t* session, const char* UNUSED(name),
-                     girara_setting_type_t UNUSED(type), void* value, void* UNUSED(data))
+                     girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
 {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
   g_return_if_fail(session->global.data != NULL);
 
-  bool inc_search = *(bool*) value;
+  bool inc_search = *(const bool*) value;
   girara_special_command_add(session, '/', cmd_search, inc_search, FORWARD,  NULL);
   girara_special_command_add(session, '?', cmd_search, inc_search, BACKWARD, NULL);
 }
 
+static void
+cb_sandbox_changed(girara_session_t* session, const char* UNUSED(name),
+                   girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
+{
+  g_return_if_fail(value != NULL);
+  g_return_if_fail(session != NULL);
+  g_return_if_fail(session->global.data != NULL);
+  zathura_t* zathura = session->global.data;
+
+  const char* sandbox = value;
+  if (g_strcmp0(sandbox, "none") == 0) {
+    zathura->global.sandbox = ZATHURA_SANDBOX_NONE;
+  } else if (g_strcmp0(sandbox, "normal") == 0)  {
+    zathura->global.sandbox = ZATHURA_SANDBOX_NORMAL;
+  } else if (g_strcmp0(sandbox, "strict") == 0) {
+    zathura->global.sandbox = ZATHURA_SANDBOX_STRICT;
+  } else {
+    girara_error("Invalid sandbox option");
+  }
+}
+
+static void
+cb_window_statbusbar_changed(girara_session_t* session, const char* name,
+                             girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
+{
+  g_return_if_fail(value != NULL);
+  g_return_if_fail(session != NULL);
+  g_return_if_fail(session->global.data != NULL);
+  zathura_t* zathura = session->global.data;
+
+  const bool is_window_setting = g_str_has_prefix(name, "window-");
+  if (is_window_setting) {
+    char* formatted_filename = get_formatted_filename(zathura, !is_window_setting);
+    girara_set_window_title(zathura->ui.session, formatted_filename);
+    g_free(formatted_filename);
+  } else {
+    statusbar_page_number_update(zathura);
+  }
+}
 
 void
 config_load_default(zathura_t* zathura)
@@ -114,7 +155,6 @@ config_load_default(zathura_t* zathura)
   int int_value              = 0;
   float float_value          = 0;
   bool bool_value            = false;
-  char* string_value         = NULL;
   girara_session_t* gsession = zathura->ui.session;
 
   /* mode settings */
@@ -152,6 +192,8 @@ config_load_default(zathura_t* zathura)
   girara_setting_add(gsession, "pages-per-row",         &int_value,   INT,    false, _("Number of pages per row"),  cb_page_layout_value_changed, NULL);
   int_value = 1;
   girara_setting_add(gsession, "first-page-column",     "1:2",        STRING, false, _("Column of the first page"), cb_page_layout_value_changed, NULL);
+  bool_value = false;
+  girara_setting_add(gsession, "page-right-to-left",    &bool_value,  BOOLEAN, false, _("Render pages from right to left"),  cb_page_layout_value_changed, NULL);
   float_value = 40;
   girara_setting_add(gsession, "scroll-step",           &float_value, FLOAT,  false, _("Scroll step"),              NULL, NULL);
   float_value = 40;
@@ -169,15 +211,17 @@ config_load_default(zathura_t* zathura)
   int_value = 2000;
   girara_setting_add(gsession, "jumplist-size",         &int_value,   INT,    false, _("Number of positions to remember in the jumplist"), cb_jumplist_change, NULL);
 
-  girara_setting_add(gsession, "recolor-darkcolor",      "#FFFFFF", STRING, false, _("Recoloring (dark color)"),         cb_color_change, NULL);
-  girara_setting_add(gsession, "recolor-lightcolor",     "#000000", STRING, false, _("Recoloring (light color)"),        cb_color_change, NULL);
-  girara_setting_add(gsession, "highlight-color",        NULL,      STRING, false, _("Color for highlighting"),          cb_color_change, NULL);
+  girara_setting_add(gsession, "recolor-darkcolor",      "#FFFFFF", STRING, false, _("Recoloring (dark color)"),            cb_color_change, NULL);
+  girara_setting_add(gsession, "recolor-lightcolor",     "#000000", STRING, false, _("Recoloring (light color)"),           cb_color_change, NULL);
+  girara_setting_add(gsession, "highlight-color",        NULL,      STRING, false, _("Color for highlighting"),             cb_color_change, NULL);
   girara_setting_set(gsession, "highlight-color",        "#9FBC00");
-  girara_setting_add(gsession, "highlight-active-color", NULL,      STRING, false, _("Color for highlighting (active)"), cb_color_change, NULL);
+  girara_setting_add(gsession, "highlight-fg",           NULL,      STRING, false, _("Foreground color for highlighting"),  cb_color_change, NULL);
+  girara_setting_set(gsession, "highlight-fg",           "#000000");
+  girara_setting_add(gsession, "highlight-active-color", NULL,      STRING, false, _("Color for highlighting (active)"),    cb_color_change, NULL);
   girara_setting_set(gsession, "highlight-active-color", "#00BC00");
-  girara_setting_add(gsession, "render-loading-bg",      NULL,      STRING, false, _("'Loading ...' background color"),  cb_color_change, NULL);
+  girara_setting_add(gsession, "render-loading-bg",      NULL,      STRING, false, _("'Loading ...' background color"),     cb_color_change, NULL);
   girara_setting_set(gsession, "render-loading-bg",      "#FFFFFF");
-  girara_setting_add(gsession, "render-loading-fg",      NULL,      STRING, false, _("'Loading ...' foreground color"),  cb_color_change, NULL);
+  girara_setting_add(gsession, "render-loading-fg",      NULL,      STRING, false, _("'Loading ...' foreground color"),     cb_color_change, NULL);
   girara_setting_set(gsession, "render-loading-fg",      "#000000");
 
   girara_setting_add(gsession, "index-fg",        "#DDDDDD", STRING, true, _("Index mode foreground color"), NULL, NULL);
@@ -228,33 +272,38 @@ config_load_default(zathura_t* zathura)
   bool_value = true;
   girara_setting_add(gsession, "abort-clear-search",     &bool_value,  BOOLEAN, false, _("Clear search results on abort"), NULL, NULL);
   bool_value = false;
-  girara_setting_add(gsession, "window-title-basename",  &bool_value,  BOOLEAN, false, _("Use basename of the file in the window title"), NULL, NULL);
+  girara_setting_add(gsession, "window-title-basename",  &bool_value,  BOOLEAN, false, _("Use basename of the file in the window title"), cb_window_statbusbar_changed, NULL);
   bool_value = false;
-  girara_setting_add(gsession, "window-title-home-tilde",  &bool_value,  BOOLEAN, false, _("Use ~ instead of $HOME in the filename in the window title"), NULL, NULL);
+  girara_setting_add(gsession, "window-title-home-tilde",  &bool_value,  BOOLEAN, false, _("Use ~ instead of $HOME in the filename in the window title"), cb_window_statbusbar_changed, NULL);
   bool_value = false;
-  girara_setting_add(gsession, "window-title-page",      &bool_value,  BOOLEAN, false, _("Display the page number in the window title"), NULL, NULL);
+  girara_setting_add(gsession, "window-title-page",      &bool_value,  BOOLEAN, false, _("Display the page number in the window title"), cb_window_statbusbar_changed, NULL);
   bool_value = false;
-  girara_setting_add(gsession, "statusbar-basename",     &bool_value,  BOOLEAN, false, _("Use basename of the file in the statusbar"), NULL, NULL);
+  girara_setting_add(gsession, "window-icon-document",   &bool_value,  BOOLEAN, false, _("Use first page of a document as window icon"), NULL, NULL);
   bool_value = false;
-  girara_setting_add(gsession, "statusbar-home-tilde",  &bool_value,  BOOLEAN, false, _("Use ~ instead of $HOME in the filename in the statusbar"), NULL, NULL);
+  girara_setting_add(gsession, "statusbar-basename",     &bool_value,  BOOLEAN, false, _("Use basename of the file in the statusbar"), cb_window_statbusbar_changed, NULL);
+  bool_value = false;
+  girara_setting_add(gsession, "statusbar-home-tilde",  &bool_value,   BOOLEAN, false, _("Use ~ instead of $HOME in the filename in the statusbar"), cb_window_statbusbar_changed, NULL);
   bool_value = true;
   girara_setting_add(gsession, "synctex",                &bool_value,  BOOLEAN, false, _("Enable synctex support"), NULL, NULL);
-  string_value = "";
-  girara_setting_add(gsession, "synctex-editor-command", string_value, STRING,  false, _("Synctex editor command"), NULL, NULL);
+  girara_setting_add(gsession, "synctex-editor-command", "",           STRING,  false, _("Synctex editor command"), NULL, NULL);
   bool_value = true;
   girara_setting_add(gsession, "dbus-service",           &bool_value,  BOOLEAN, false, _("Enable D-Bus service"), NULL, NULL);
+  girara_setting_add(gsession, "dbus-raise-window",      &bool_value,  BOOLEAN, false, _("Raise window on certain D-Bus commands"), NULL, NULL);
   bool_value = false;
   girara_setting_add(gsession, "continuous-hist-save",   &bool_value,  BOOLEAN, false, _("Save history at each page change"), NULL, NULL);
-  string_value = "primary";
-  girara_setting_add(gsession, "selection-clipboard",    string_value, STRING,  false, _("The clipboard into which mouse-selected data will be written"), NULL, NULL);
+  girara_setting_add(gsession, "selection-clipboard",    "primary",    STRING,  false, _("The clipboard into which mouse-selected data will be written"), NULL, NULL);
   bool_value = true;
   girara_setting_add(gsession, "selection-notification", &bool_value,  BOOLEAN, false, _("Enable notification after selecting text"), NULL, NULL);
+  /* default to no sandbox when running in WSL */
+  const char* string_value = running_under_wsl() ? "none" : "normal";
+  girara_setting_add(gsession, "sandbox",                string_value, STRING, true,   _("Sandbox level"), cb_sandbox_changed, NULL);
 
 #define DEFAULT_SHORTCUTS(mode) \
   girara_shortcut_add(gsession, 0,                GDK_KEY_a,          NULL, sc_adjust_window,           (mode),     ZATHURA_ADJUST_BESTFIT, NULL); \
   girara_shortcut_add(gsession, 0,                GDK_KEY_s,          NULL, sc_adjust_window,           (mode),     ZATHURA_ADJUST_WIDTH,   NULL); \
 \
   girara_shortcut_add(gsession, 0,                GDK_KEY_F,          NULL, sc_display_link,            (mode),     0,                      NULL); \
+  girara_shortcut_add(gsession, 0,                GDK_KEY_c,          NULL, sc_copy_link,               (mode),     0,                      NULL); \
 \
   girara_shortcut_add(gsession, 0,                GDK_KEY_slash,      NULL, sc_focus_inputbar,          (mode),     0,                      &("/")); \
   girara_shortcut_add(gsession, GDK_SHIFT_MASK,   GDK_KEY_slash,      NULL, sc_focus_inputbar,          (mode),     0,                      &("/")); \
@@ -315,6 +364,8 @@ config_load_default(zathura_t* zathura)
 \
   girara_shortcut_add(gsession, 0,                GDK_KEY_n,          NULL, sc_search,                  (mode),     FORWARD,                NULL); \
   girara_shortcut_add(gsession, 0,                GDK_KEY_N,          NULL, sc_search,                  (mode),     BACKWARD,               NULL); \
+\
+  girara_shortcut_add(gsession, 0,                GDK_KEY_P,          NULL, sc_snap_to_page,            (mode),     0,                      NULL); \
 \
   girara_shortcut_add(gsession, 0,                GDK_KEY_Tab,        NULL, sc_toggle_index,            (mode),     0,                      NULL); \
   girara_shortcut_add(gsession, GDK_CONTROL_MASK, GDK_KEY_n,          NULL, girara_sc_toggle_statusbar, (mode),     0,                      NULL); \
@@ -448,6 +499,8 @@ config_load_default(zathura_t* zathura)
   girara_inputbar_command_add(gsession, "open",       "o",    cmd_open,            cc_open,      _("Open document"));
   girara_inputbar_command_add(gsession, "quit",       "q",    cmd_quit,            NULL,         _("Close zathura"));
   girara_inputbar_command_add(gsession, "print",      NULL,   cmd_print,           NULL,         _("Print document"));
+  girara_inputbar_command_add(gsession, "save",       NULL,   cmd_save,            cc_write,     _("Save document"));
+  girara_inputbar_command_add(gsession, "save!",      NULL,   cmd_savef,           cc_write,     _("Save document (and force overwriting)"));
   girara_inputbar_command_add(gsession, "write",      NULL,   cmd_save,            cc_write,     _("Save document"));
   girara_inputbar_command_add(gsession, "write!",     NULL,   cmd_savef,           cc_write,     _("Save document (and force overwriting)"));
   girara_inputbar_command_add(gsession, "export",     NULL,   cmd_export,          cc_export,    _("Save attachments"));
@@ -467,6 +520,7 @@ config_load_default(zathura_t* zathura)
   girara_shortcut_mapping_add(gsession, "bisect",              sc_bisect);
   girara_shortcut_mapping_add(gsession, "change_mode",         sc_change_mode);
   girara_shortcut_mapping_add(gsession, "display_link",        sc_display_link);
+  girara_shortcut_mapping_add(gsession, "copy_link",           sc_copy_link);
   girara_shortcut_mapping_add(gsession, "exec",                sc_exec);
   girara_shortcut_mapping_add(gsession, "focus_inputbar",      sc_focus_inputbar);
   girara_shortcut_mapping_add(gsession, "follow",              sc_follow);
@@ -476,6 +530,7 @@ config_load_default(zathura_t* zathura)
   girara_shortcut_mapping_add(gsession, "mark_evaluate",       sc_mark_evaluate);
   girara_shortcut_mapping_add(gsession, "navigate",            sc_navigate);
   girara_shortcut_mapping_add(gsession, "navigate_index",      sc_navigate_index);
+  girara_shortcut_mapping_add(gsession, "nohlsearch",          sc_nohlsearch);
   girara_shortcut_mapping_add(gsession, "print",               sc_print);
   girara_shortcut_mapping_add(gsession, "quit",                sc_quit);
   girara_shortcut_mapping_add(gsession, "recolor",             sc_recolor);
@@ -483,6 +538,7 @@ config_load_default(zathura_t* zathura)
   girara_shortcut_mapping_add(gsession, "rotate",              sc_rotate);
   girara_shortcut_mapping_add(gsession, "scroll",              sc_scroll);
   girara_shortcut_mapping_add(gsession, "search",              sc_search);
+  girara_shortcut_mapping_add(gsession, "snap_to_page",        sc_snap_to_page);
   girara_shortcut_mapping_add(gsession, "toggle_fullscreen",   sc_toggle_fullscreen);
   girara_shortcut_mapping_add(gsession, "toggle_index",        sc_toggle_index);
   girara_shortcut_mapping_add(gsession, "toggle_page_mode",    sc_toggle_page_mode);
