@@ -1,4 +1,4 @@
-/* See LICENSE file for license and copyright information */
+/* SPDX-License-Identifier: Zlib */
 
 #include <girara/session.h>
 #include <girara/settings.h>
@@ -149,6 +149,31 @@ sc_display_link(girara_session_t* session, girara_argument_t* UNUSED(argument),
   return false;
 }
 
+bool
+sc_copy_link(girara_session_t* session, girara_argument_t* UNUSED(argument),
+             girara_event_t* UNUSED(event), unsigned int UNUSED(t))
+{
+  g_return_val_if_fail(session != NULL, false);
+  g_return_val_if_fail(session->global.data != NULL, false);
+  zathura_t* zathura = session->global.data;
+
+  if (zathura->document == NULL || zathura->ui.session == NULL) {
+    return false;
+  }
+
+  bool show_links = draw_links(zathura);
+
+  /* ask for input */
+  if (show_links) {
+    zathura_document_set_adjust_mode(zathura->document, ZATHURA_ADJUST_INPUTBAR);
+    girara_dialog(zathura->ui.session, "Copy link:", FALSE, NULL,
+        cb_sc_copy_link,
+        zathura->ui.session);
+  }
+
+  return false;
+}
+
 bool
 sc_focus_inputbar(girara_session_t* session, girara_argument_t* argument, girara_event_t* UNUSED(event), unsigned int UNUSED(t))
 {
@@ -562,8 +587,9 @@ sc_scroll(girara_session_t* session, girara_argument_t* argument,
     direction = -1.0;
   }
 
-  const double vstep = (double)view_height / (double)doc_height;
-  const double hstep = (double)view_width / (double)doc_width;
+  unsigned int pad  = zathura_document_get_page_padding(zathura->document);
+  const double vstep = (double)(view_height + pad) / (double)doc_height;
+  const double hstep = (double)(view_width + pad) / (double)doc_width;
 
   /* compute new position */
   switch (argument->n) {
@@ -751,7 +777,7 @@ sc_bisect(girara_session_t* session, girara_argument_t* argument,
       zathura->bisect.end = cur_page;
       direction = FORWARD;
     }
-  } else if (argument != NULL)  {
+  } else {
     direction = argument->n;
 
     /* setup initial bisect range */
@@ -760,7 +786,6 @@ sc_bisect(girara_session_t* session, girara_argument_t* argument,
       girara_debug("bisecting between first and last page because there are no jumps");
       zathura->bisect.start = 0;
       zathura->bisect.end = num_pages - 1;
-
     } else if (jump->page != cur_page || jump->page != zathura->bisect.last_jump) {
       girara_debug("last jump doesn't match up, starting new bisecting");
       zathura->bisect.start = 0;
@@ -787,8 +812,6 @@ sc_bisect(girara_session_t* session, girara_argument_t* argument,
       zathura->bisect.end = MAX(prev_page, cur_page);
       zathura->bisect.last_jump = cur_page;
     }
-  } else {
-    return false;
   }
 
   girara_debug("bisecting between %d and %d, at %d", zathura->bisect.start, zathura->bisect.end, cur_page);
@@ -912,7 +935,7 @@ sc_search(girara_session_t* session, girara_argument_t* argument,
       /* the next result is on a different page */
       g_object_set(G_OBJECT(page_widget), "search-current", -1, NULL);
 
-      for (int npage_id = 1; page_id < num_pages; ++npage_id) {
+      for (unsigned int npage_id = 1; npage_id < num_pages; ++npage_id) {
         int ntmp = cur_page + diff * (page_id + npage_id);
         zathura_page_t* npage = zathura_document_get_page(zathura->document, (ntmp + 2*num_pages) % num_pages);
         GtkWidget* npage_page_widget = zathura_page_get_widget(zathura, npage);
@@ -960,11 +983,11 @@ sc_search(girara_session_t* session, girara_argument_t* argument,
 
     /* compute the center of the rectangle, which will be aligned to the center
        of the viewport */
-    double center_x = (rectangle.x1 + rectangle.x2) / 2;
-    double center_y = (rectangle.y1 + rectangle.y2) / 2;
-
+    const double center_y = (rectangle.y1 + rectangle.y2) / 2;
     pos_y += (center_y - (double)cell_height/2) / (double)doc_height;
+
     if (search_hadjust == true) {
+      const double center_x = (rectangle.x1 + rectangle.x2) / 2;
       pos_x += (center_x - (double)cell_width/2) / (double)doc_width;
     }
 
@@ -972,9 +995,11 @@ sc_search(girara_session_t* session, girara_argument_t* argument,
     zathura_jumplist_add(zathura);
     position_set(zathura, pos_x, pos_y);
     zathura_jumplist_add(zathura);
-  } else if (target_page == NULL && argument->data != NULL) {
+  } else if (argument->data != NULL) {
     const char* input = argument->data;
-    girara_notify(session, GIRARA_ERROR, _("Pattern not found: %s"), input);
+    char* escaped_text = g_markup_printf_escaped(_("Pattern not found: %s"), input);
+    girara_notify(session, GIRARA_ERROR, "%s", escaped_text);
+    g_free(escaped_text);
   }
 
   return false;
@@ -1190,6 +1215,14 @@ sc_toggle_index(girara_session_t* session, girara_argument_t* UNUSED(argument),
     /* save current position to the jumplist */
     zathura_jumplist_add(zathura);
 
+    const zathura_adjust_mode_t adjust_mode =
+      zathura_document_get_adjust_mode(zathura->document);
+
+    /* zathura goes to the first page when toggling index mode if this isn't done */
+    if (adjust_mode == ZATHURA_ADJUST_INPUTBAR) {
+      zathura_document_set_adjust_mode(zathura->document, ZATHURA_ADJUST_NONE);
+    }
+
     girara_set_view(session, zathura->ui.index);
     gtk_widget_show(GTK_WIDGET(zathura->ui.index));
     girara_mode_set(zathura->ui.session, zathura->modes.index);
@@ -1304,8 +1337,8 @@ sc_toggle_presentation(girara_session_t* session, girara_argument_t*
     /* set full screen */
     gtk_window_unfullscreen(GTK_WINDOW(session->gtk.window));
 
-    /* reset scale */
-    zathura_document_set_scale(zathura->document, zathura->shortcut.toggle_presentation_mode.zoom);
+    /* reset zoom */
+    zathura_document_set_zoom(zathura->document, zathura->shortcut.toggle_presentation_mode.zoom);
     render_all(zathura);
     refresh_view(zathura);
 
@@ -1326,7 +1359,7 @@ sc_toggle_presentation(girara_session_t* session, girara_argument_t*
     girara_setting_set(session, "pages-per-row", &int_value);
 
     /* back up zoom */
-    zathura->shortcut.toggle_presentation_mode.zoom = zathura_document_get_scale(zathura->document);
+    zathura->shortcut.toggle_presentation_mode.zoom = zathura_document_get_zoom(zathura->document);
 
     /* adjust window */
     girara_argument_t argument = { ZATHURA_ADJUST_BESTFIT, NULL };
@@ -1379,43 +1412,44 @@ sc_zoom(girara_session_t* session, girara_argument_t* argument, girara_event_t*
 
   const int nt = (t == 0) ? 1 : t;
   const double zoom_step = 1.0 + value / 100.0 * nt;
-  const double old_zoom = zathura_document_get_scale(zathura->document);
+  const double old_zoom = zathura_document_get_zoom(zathura->document);
 
   /* specify new zoom value */
   if (argument->n == ZOOM_IN) {
-    girara_debug("Increasing zoom by %f.", zoom_step - 1.0);
-    zathura_document_set_scale(zathura->document, old_zoom * zoom_step);
+    girara_debug("Increasing zoom by %0.2f.", zoom_step - 1.0);
+    zathura_document_set_zoom(zathura->document, old_zoom * zoom_step);
   } else if (argument->n == ZOOM_OUT) {
-    girara_debug("Decreasing zoom by %f.", zoom_step - 1.0);
-    zathura_document_set_scale(zathura->document, old_zoom / zoom_step);
+    girara_debug("Decreasing zoom by %0.2f.", zoom_step - 1.0);
+    zathura_document_set_zoom(zathura->document, old_zoom / zoom_step);
   } else if (argument->n == ZOOM_SPECIFIC) {
     if (t == 0) {
       girara_debug("Setting zoom to 1.");
-      zathura_document_set_scale(zathura->document, 1.0);
+      zathura_document_set_zoom(zathura->document, 1.0);
     } else {
-      girara_debug("Setting zoom to %f.", t / 100.0);
-      zathura_document_set_scale(zathura->document, t / 100.0);
+      girara_debug("Setting zoom to %0.2f.", t / 100.0);
+      zathura_document_set_zoom(zathura->document, t / 100.0);
     }
   } else if (argument->n == ZOOM_SMOOTH) {
     const double dy = (event != NULL) ? event->y : 1.0;
-    girara_debug("Increasing zoom by %f.", zoom_step * dy - 1.0);
-    zathura_document_set_scale(zathura->document, old_zoom + zoom_step * dy);
+    const double z = pow(zoom_step, -dy);
+    girara_debug("Increasing zoom by %0.2f.",  z - 1.0);
+    zathura_document_set_zoom(zathura->document, old_zoom * z);
   } else {
     girara_debug("Setting zoom to 1.");
-    zathura_document_set_scale(zathura->document, 1.0);
+    zathura_document_set_zoom(zathura->document, 1.0);
   }
 
   /* zoom limitations */
-  const double scale = zathura_document_get_scale(zathura->document);
-  zathura_document_set_scale(zathura->document, zathura_correct_scale_value(session, scale));
+  const double zoom = zathura_document_get_zoom(zathura->document);
+  zathura_document_set_zoom(zathura->document, zathura_correct_zoom_value(session, zoom));
 
-  const double new_zoom = zathura_document_get_scale(zathura->document);
+  const double new_zoom = zathura_document_get_zoom(zathura->document);
   if (fabs(new_zoom - old_zoom) <= DBL_EPSILON) {
-    girara_debug("New and old zoom level are too close: %f vs. %f, diff = %f", new_zoom, old_zoom, fabs(new_zoom - old_zoom));
+    girara_debug("New and old zoom level are too close: %0.2f vs. %0.2f", new_zoom, old_zoom);
     return false;
   }
 
-  girara_debug("Re-rendering with new zoom level %f.", new_zoom);
+  girara_debug("Re-rendering with new zoom level %0.2f.", new_zoom);
   render_all(zathura);
   refresh_view(zathura);
 
@@ -1458,3 +1492,29 @@ sc_exec(girara_session_t* session, girara_argument_t* argument, girara_event_t*
 
   return girara_sc_exec(session, argument, event, t);
 }
+
+bool
+sc_nohlsearch(girara_session_t* session, girara_argument_t* UNUSED(argument), girara_event_t* UNUSED(event), unsigned int UNUSED(t))
+{
+  g_return_val_if_fail(session != NULL, false);
+  g_return_val_if_fail(session->global.data != NULL, false);
+  zathura_t* zathura = session->global.data;
+
+  document_draw_search_results(zathura, false);
+  render_all(zathura);
+
+  return false;
+}
+
+bool sc_snap_to_page(girara_session_t *session,
+                     girara_argument_t *UNUSED(argument),
+                     girara_event_t *UNUSED(event), unsigned int UNUSED(t)) {
+  g_return_val_if_fail(session != NULL, false);
+  g_return_val_if_fail(session->global.data != NULL, false);
+  zathura_t *zathura = session->global.data;
+  g_return_val_if_fail(zathura->document != NULL, false);
+  zathura_document_t *document = zathura->document;
+
+  int page = zathura_document_get_current_page_number(document);
+  return page_set(zathura, page);
+}
